/**
 *
 * Helsinki Security Meetup, 20th Aug 2014
 * Using SElinux: Is it insane, too much trouble or the only smart thing to do?
 *
 * The URL for this material is 
 * http://blog.hqcodeshop.fi/archives/223-Helsinki-Security-Meetup-SElinux-presentation.html
 *
 * Public domain by Jari Turkia. Author appreciates any credits to his work.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>


#define DEFAULT_PORT 8282

void main ()
{
int sock, fd, len, n;
struct sockaddr_in server;
char buff[1024], c;
char *cmd = "/bin/bash";
char *cmd_args[] = { "/bin/bash",
                       "-i",
                       NULL }; /* note: last item is NULL */
char *greeting = "Hello there! Welcome to my box!\n";

  /* Create a new socket */
  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0) {
    perror ("opening socket");
    exit(1);
  }

  /* We're binding into IP-address of INADDR_ANY. */
  server.sin_family = AF_INET;
  server.sin_addr.s_addr = INADDR_ANY;
  server.sin_port = htons (DEFAULT_PORT);

  /* We're binding into a specific TCP-port */
  len = sizeof(server);
  if (bind(sock, (struct sockaddr *)&server, len) < 0) {
    perror ("binding");
    exit(1);
  }

  /* Inform, that we're ready */
  if (getsockname(sock, (struct sockaddr *)&server, &len) < 0) {
    perror ("getting name");
    exit(1);
  }
  printf("Listening to TCP/%d for connections.\n",
    ntohs(server.sin_port));

  /* Start listening the socket. Create a queue of 5 requests. */
  listen(sock, 5);
  while (1) {
    /* accept incoming connection, block until one is available, assign socket fd */
    fd = accept(sock, (struct sockaddr *)0, (int *)0);
    if (fd == -1) {
      perror("accept fd");
      exit(1);
    }

    /* child process ID from fork() */
    pid_t cpid = fork();
    if (cpid < 0)
      exit(1);  /* exit if fork() fails */
    if (cpid) {
      /* In the parent process: */
      printf("The child process is in PID %d\n", cpid);
      close(fd);                /* fd is not needed in the parent after the fork */
      waitpid(cpid, NULL, 0);   /* wait for and reap child process */
      printf("Child process PID %d exited\n", cpid);
    } else {
      /* In the child process: */
      write(fd, greeting, strlen(greeting));

      /* Store stdout for case of failure */
      int stdout_temp = dup(STDOUT_FILENO);

      dup2(fd, STDIN_FILENO);   /* duplicate socket on stdin */
      dup2(fd, STDOUT_FILENO);  /* duplicate socket on stdout */
      dup2(fd, STDERR_FILENO);  /* duplicate socket on stderr too */
      close(fd);                /* can close the original after it's duplicated */
      execvp(cmd, cmd_args);    /* execvp() the command */

      /* This won't be reached, unless execvp() failed */
      printf("Argh! I failed :-(\n");           /* This goes out to remote client */
      dup2(stdout_temp, STDOUT_FILENO);
      printf("Child couldn't execvp()!\n");     /* This goes out to tty */
      exit(1);
    }
  } /* end while (1) */
}
